package com.ekexiu.console.system.service;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Delete;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.util.StringUtil;
import org.jfw.util.auth.AuthUtil;

import com.ekexiu.console.system.dao.RoleDao;
import com.ekexiu.console.system.dao.UserRoleDao;
import com.ekexiu.console.system.po.Role;
import com.ekexiu.console.system.vo.ConsoleAuthRole;

@Path("/sys/role")
public class RoleService {
	
	public static final String EMPTY_AUTH_STR = AuthUtil.serialAuth(new int[]{0});
	
	@Autowrie
	private RoleDao roleDao;
	@Autowrie
	private UserRoleDao userRoleDao;

	public UserRoleDao getUserRoleDao() {
		return userRoleDao;
	}

	public void setUserRoleDao(UserRoleDao userRoleDao) {
		this.userRoleDao = userRoleDao;
	}

	public RoleDao getRoleDao() {
		return roleDao;
	}

	public void setRoleDao(RoleDao roleDao) {
		this.roleDao = roleDao;
	}
	
	
	
	@Post
	@Path
	public void insert(@JdbcConn(true) Connection con,ConsoleAuthRole role)throws SQLException{
		Role r = new Role();
		r.setId(StringUtil.buildUUID());
		r.setName(role.getName());
		r.setDescp(role.getDescp());
		int[] auths = role.getAuths();
		if(auths==null || auths.length==0){
			r.setAuthinfo(EMPTY_AUTH_STR);
		}else{
			r.setAuthinfo(AuthUtil.serialAuth(AuthUtil.compress(auths)));
		}
		this.roleDao.insert(con, r);
	}
	@Put
	@Path
	public void update(@JdbcConn(true) Connection con,@RequestBody ConsoleAuthRole role)throws SQLException{
		Role r = new Role();
		r.setName(role.getName());
		r.setDescp(role.getDescp());
		int[] auths = role.getAuths();
		if(auths==null || auths.length==0){
			r.setAuthinfo(EMPTY_AUTH_STR);
		}else{
			r.setAuthinfo(AuthUtil.serialAuth(AuthUtil.compress(auths)));
		}
		this.roleDao.update(con, r);
	}
	
	@Path("/{id}")
	@Delete
	public void delete(@JdbcConn(true) Connection con,@PathVar String id)throws SQLException{
		 this.userRoleDao.deleteByRoleId(con, id);		 
		 this.roleDao.delete(con, id);
	}
	
	@Path("/all")
	@Get
	public List<ConsoleAuthRole> queryAll(@JdbcConn Connection con)throws SQLException{
		List<Role> roles= this.roleDao.query(con);
		ArrayList<ConsoleAuthRole> ret = new ArrayList<ConsoleAuthRole>(roles.size());
		for(ListIterator<Role> it = roles.listIterator(); it.hasNext();){
			ret.add(conver(it.next()));
		}
		return ret;
	}
	@Path("/id/{id}")
	@Get
	public ConsoleAuthRole queryById(@JdbcConn Connection con ,@PathVar String id ) throws SQLException{
	  return conver(this.roleDao.query(con, id));
	}
	@Path("/byUserid/{userid}")
	@Get
	public List<ConsoleAuthRole> queryByUserId(@JdbcConn Connection con,@PathVar String userid)throws SQLException{
		List<Role> roles= this.roleDao.queryByUser(con, userid);
		ArrayList<ConsoleAuthRole> ret = new ArrayList<ConsoleAuthRole>(roles.size());
		for(ListIterator<Role> it = roles.listIterator(); it.hasNext();){
			ret.add(conver(it.next()));
		}
		return ret;
	}

	
	
	
	
	
	
	
	
	public static ConsoleAuthRole conver(Role role){
		if(null == role) return null;
		ConsoleAuthRole cr = new ConsoleAuthRole();
		cr.setDescp(role.getDescp());
		cr.setId(role.getId());
		cr.setName(role.getName());
		cr.setAuths(AuthUtil.unCompress(AuthUtil.deSerialAuth(role.getAuthinfo())));
		return cr;
	}
}
